-- Base class for Waypoints

local AceOO = AceLibrary("AceOO-2.0")

local Crayon = AceLibrary("Crayon-2.0")

-- All Waypoints require the following function
local IWaypoint = AceOO.Interface { 
	Location = "function",  -- Returns location info of the waypoint
	ToString = "function",	-- Returns the name shown on the waypoint arrow
	Cancel = "function",	-- A function to clear the waypoint info
	GetDistance = "function", -- A function to calculate the current distance from the waypoint
	IsValid = "function",	-- Is the waypoint still valid?
}

Waypoint = AceOO.Class(IWaypoint)
Waypoint.virtual = true -- Virtual class

-- Pool of frames to reuse.
Waypoint.framePool = setmetatable({}, 
						( {__index = function(self, n)
										if n==0 then
											return table.remove(self) or {}
										end
									end})
						)

local function getFrame() 
	return Waypoint.framePool[0]
end
						
local function returnFrame(frame)
	table.insert(Waypoint.framePool, frame)
end

Waypoint.prototype.WaypointID = -1		-- Unique ID
Waypoint.prototype.x = -1				-- Current X co-ord of waypoint
Waypoint.prototype.y = -1				-- Current Y co-ord of waypoint
Waypoint.prototype.Zone = "Limbo"		-- Current localised zone of waypoint
Waypoint.prototype.EZone = "Void"		-- Current english zone of waypoint
-- Graphics 
Waypoint.prototype.Frame = getFrame()	-- Frame to show waypoint
Waypoint.prototype.Colour = Crayon.COLOR_HEX_GREEN

function Waypoint.prototype:init(id)
	Waypoint.super.prototype.init(self)
	self.WaypointID = id
end

-- This function will return the x and y position relative 
-- to the map along with the zone and english zone name.
function Waypoint.prototype:Location()
	return self.x, self.y, self.Zone, self.EZone
end

-- Use Cartographer to find the distance
function Waypoint.prototype:GetDistance()
	return Cartographer:GetDistanceToPoint(self.x, self.y, self.Zone)
end

-- This function clears out values specific to the class
function Waypoint.prototype:Cancel()
	self.WaypointID, self.x, self.y, self.Zone, self.EZone = nil
	returnFrame(self.Frame)
end

-- Default ToString reverts to the ID
function Waypoint.prototype:ToString()
	return self.WaypointID
end

-- Default check to see if waypoint is ready
function Waypoint.prototype:IsReady()
	return true
end

-- Check if waypoint is valid 
function Waypoint.prototype:IsValid()
	return true
end

-- Lets give a generic frame
